# A command line BitTorrent client.

I started writing a BitTorrent client because it seemed like fun and I am learning Haskell and wanted to see if I can write something real with Haskell than just heat the room. It is turning out to be a lot of fun.

### Building

You need to install [Stack](https://github.com/commercialhaskell/stack), either via
the OS package manager or via `cabal` (`cabal install stack`).

    $ git clone https://github.com/vu3rdd/functorrent && cd functorrent
    $ cabal install stack # or install stack by other means
    $ stack build # binaries in $(pwd)/.stack-work/install/x86..../lts-2.16/7.8.4/bin/functorrent

### Usage

If you invoke functorrent without any options, it expects the contents of a torrent file
to be given in stdin. So

    $ cat ubuntu-14.10-desktop-amd64.iso.torrent | functorrent
    [....]

Or one can explicitly specify the torrent file as input.

    $ functorrent ubuntu-14.10-desktop-amd64.iso.torrent
    [...]

### Goals

- [Optimized for Fun](http://www.slideshare.net/autang/ofun-optimizing-for-fun).
  (should have called it "funtorrent")
- Become more profient with Haskell.
- Implement something non-trivial with Haskell (crypto, file operations, network
  operations, concurrency, bit twiddling, DHT).
- Follow the spec ([official spec](http://bittorrent.org/beps/bep_0003.html), [unofficial spec](https://wiki.theory.org/BitTorrentSpecification))
- Easy for newbies like me to read and understand alongside the spec.
- doctest and quickcheck tests.
- Follow Haskell Style Guide - https://github.com/tibbe/haskell-style-guide/blob/master/haskell-style.md

### Current Status

- most of the peer wire protocol works.
- talks only to one peer at the moment.
- can download files. But needs every piece to be served by the peer it connected to.

### WARNING

This client is not usable as your daily BitTorrent client yet. You may get corrupted files and end up wasting a lot of bandwidth. So, until we achieve some robustness, consider this as a programmer-friendly project to learn about a bunch of stuff.

## TODO

*    Test suite.
*    Talk to multiple peers concurrently.
*    Piece download algorithms.
*    Multifile torrent support.
*    other advanced features of BitTorrent (like DHT).

### Misc

The Bangalore Haskell group forked an early version of this code and started to develop it as a group. My code has diverged a lot from it. Both are calling it functorrent. Perhaps that needs to be fixed.
