{-
 - Copyright (C) 2015-2016 Ramakrishnan Muthukrishnan <ram@rkrishnan.org>
 -
 - This file is part of FuncTorrent.
 -
 - FuncTorrent is free software; you can redistribute it and/or modify
 - it under the terms of the GNU General Public License as published by
 - the Free Software Foundation; either version 3 of the License, or
 - (at your option) any later version.
 -
 - FuncTorrent is distributed in the hope that it will be useful,
 - but WITHOUT ANY WARRANTY; without even the implied warranty of
 - MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 - GNU General Public License for more details.
 -
 - You should have received a copy of the GNU General Public License
 - along with FuncTorrent; if not,  see <http://www.gnu.org/licenses/>
 -}

{-# LANGUAGE OverloadedStrings #-}

module FuncTorrent.Server where

import Control.Concurrent (forkIO)
import Control.Monad (forever)
import Network (withSocketsDo, listenOn, accept, Socket, PortID ( PortNumber ))
import System.IO (hSetBuffering, BufferMode ( NoBuffering ))

import FuncTorrent.Metainfo (Metainfo)
import FuncTorrent.Peer (handlePeerMsgs, PieceMap)
import FuncTorrent.PeerMsgs (Peer(..))
import qualified FuncTorrent.FileSystem as FS (MsgChannel)

-- server is listening on any port from 6881 - 6889
-- return the port number used
start :: IO (Socket, PortID)
start = withSocketsDo $ do
  let portnums = [6881 .. 6889]
  sock <- listenOn $ PortNumber $ fromIntegral (head portnums)
  return (sock, PortNumber $ head portnums)

run :: Socket -> String -> Metainfo -> PieceMap -> FS.MsgChannel -> IO ()
run listenSock peerid m pieceMap c = forever $ do
  (handle, ip, port) <- accept listenSock
  let peer = Peer ip (fromIntegral port)
  hSetBuffering handle NoBuffering
  forkIO $ handlePeerMsgs peer peerid m pieceMap False c
